<?php
/**
 * SkeletonCart
 * (c) 2020 Juxtaposition Inc. All Rights Reserved.
 * (c) 2020 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace StripeSimple;

use App\CharCodeChecker;
use Stripe\Stripe;
use Stripe\Exception\ExceptionInterface;

require_once('lib/stripe-php/init.php');

class Plugin extends \App\Plugin
{
    protected $priceList = array();
    protected $intent;
    public $config;

    public function __construct($key)
    {
        parent::__construct($key);

        $fileName = __DIR__ . "/Config.php";
        if (!is_readable($fileName)) {
            throw new \RuntimeException("設定ファイルが読み込めません。");
        }
        $this->config = include $fileName;

        $this->initCallbacks();
    }

    public function initCallbacks()
    {
        \App::addCallback('preStart', array($this, 'preStart'));
        \App::addCallback('preRender', array($this, 'preRender'));
        \App::addCallback('postInitRoute', array($this, 'postInitRoute'));
    }

    public function getProductDescription()
    {
        $max_len = 16;
        $post_fix = '、他';
        $omit = '...';

        $cart = \App::cart();
        $item = $cart[0];
        $description = $item->name();
        if(mb_strlen($description) > $max_len) {
            $description = mb_substr($description, 0, $max_len) . $omit;
        }
        if($cart->count() > 1) {
            $description .= $post_fix;
        }
        $description = CharCodeChecker::filterJISX0208($description);
        return $description;
    }

    public function preStart($controller)
    {
        if($controller->getType() != 'checkout' || $controller->getMode() != 'complete') {
            return;
        }

        $slip = \App::slip();
        $payment_ways = is_array($this->config['payment_way'])
            ? $this->config['payment_way']
            : [ $this->config['payment_way'] => 'card' ];


        if(!array_key_exists($slip->get('payment_way'), $payment_ways)) {
            return;
        }

        $params = [
            'amount' => $slip->payment(),
            'currency' => 'jpy',
            'receipt_email' => $slip->get('user_mail'),
            'metadata' => [
                'order_id' => $slip->get('id'),
                'client' => 'skeletoncart',
            ],
        ];

        $payment_method_type = $payment_ways[$slip->get('payment_way')];
        if($payment_method_type === 'konbini') {
            $params['payment_method_types'] = [ $payment_method_type ];
            $params['payment_method_options'] = [
                'konbini' => [
                    'expires_after_days' => $this->config['expires_after_days'],
                    'product_description' => $this->getProductDescription(),
                ],
            ];
        }
        else {
            $params['payment_method_types'] = [ $payment_method_type ];
        }

        \Stripe\Stripe::setApiKey($this->config['secretApiKey']);

        try {
            $intent = \Stripe\PaymentIntent::create($params);
            $this->intent = $intent;
        }
        catch(ExceptionInterface $ex) {
            \App::logger()->error(__METHOD__ . ': ' . get_class($ex) . ': ' . $ex->getMessage());
            \App::logger()->debug(__METHOD__ . ": " . $ex->getTraceAsString());
            throw $ex;
        }
    }

    public function preRender($template, $data)
    {
        if($template == '_regi_comp.html' && $this->intent) {
            // Add CSP header
            if(!headers_sent()) {
                $policy = join('; ', [
                    'connect-src https://api.stripe.com',
                    'frame-src https://js.stripe.com https://hooks.stripe.com',
                ]);
                header('Content-Security-Policy: ' . $policy);
            }

            $script = <<<'HTML'
<script type="text/javascript">
StripeSimpleSettings = {
    apiKey: '{$stripe_api_key}',
    clientSecret: '{$stripe_client_secret}',
    userMail: '{$user_mail}',
    userName: '{$user_name}',
    userTel: '{$user_tel}'
}
</script>
HTML;
            \App::view()->assign([
                'stripe_api_key' => $this->config['publicApiKey'],
                'stripe_client_secret' => $this->intent->client_secret,
            ]);
            $script_out = \App::view()->fetch('string:'.$script);

            $html_head = \App::view()->getTemplateVars('html_head');
            $html_head .= $script_out;
            \App::view()->assign('html_head', $html_head);

            $html_foot = '';
            \App::view()->assign('html_foot', $html_foot);
        }
    }

    public function postInitRoute()
    {
        \App::route(
            'POST /webhook/test',
            function () {
                return \App::json(['result' => true], 200);
            },
            false
        );
        \App::route(
            'POST /webhook/stripe',
            function () {
                $controller = new WebhookController();
                $controller->start();   
            },
            false
        );
    }
}

\App::addPlugin(new Plugin('StripeSimple'));

